import { Meteor } from 'meteor/meteor';
import { Mongo } from 'meteor/mongo';
import { check } from 'meteor/check';

import { Goals } from '../models/goal.js';
import { Tasks } from '../models/task.js';

Meteor.methods({
  'goals.insert'(task) {
    check(task.name, String);

    Goals.insert({
      name: task.name,
      isGroup : task.isGroup,
      score : task.score,
      iconIndex : task.iconIndex,
      age : task.age
    });

  },
'goals.remove'(goalId) {
    Goals.remove({ _id : goalId});
  },
  'goals.assignTo'(goalId, personId){
    var goal = Goals.findOne(goalId);
    if (!goal.isAssigned){
      // clone taks
      Goals.remove(
        { personId : { $eq : personId },  isAssigned : { $eq: true }, isCompleted : { $ne : true } },
      );

      Goals.insert({
        name: goal.name,
        isGroup : goal.isGroup,
        score : goal.score,
        value : goal.value,
        iconIndex : goal.iconIndex,
        personId : personId,
        age : goal.age,
        goalId : goal._id,
        isAssigned : true
      });
    }
  },
  'goals.assignToGroup'(goalId, groupId){
    var goal = Goals.findOne(goalId);
    if (!goal.isAssigned){
      // Remove all goal assigned to groupId but not completet .. can only have one at a time

          // Update all tasks
      Goals.remove(
        { groupId : { $eq : groupId },  isAssigned : { $eq: true }, isCompleted : { $ne : true } },
      );

      // clone taks
      Goals.insert({
        name: goal.name,
        isGroup : goal.isGroup,
        score : goal.score,
        value : goal.value,
        iconIndex : goal.iconIndex,
        groupId : groupId,
        age : goal.age,
        goalId : goal._id,
        isAssigned : true
      });
    }
  },
'goals.addAttributes'(id, fields) {
    Goals.update(
      {
        _id: id
      },
      {
        $set: fields
      });
  },
  'goals.isCompleted'(goalId, tasksId){
    // Find all tasks assigned to personIndex
    Goals.update(
      { _id: goalId },
      { 
        $push: { tasks: { $each: tasksId } },
        $set:  { isCompleted: true }
      }
    );

    // Update all tasks
    Tasks.update(
      { _id : { $in : tasksId } },
      {
        $set: { isGoaled: true }
      },
      {
        multi: true,
      }
    );
  }
});

if (Meteor.isServer) {
  Meteor.publish('goals', function() {
    return Goals.find({ isAssigned : {$eq: false} });
  });
  Meteor.publish('goalInstances', function() {
    return Goals.find({ isAssigned : {$eq: true}, isRedeemed : { $ne : true } });
  });
}

Meteor.startup(() => {
  if (Goals.find({}).count() === 0) {
    Goals.insert({ name : "Stjerne",      score : 50,  age : 5, iconIndex : 0 });
    Goals.insert({ name : "MultiStjerne", score : 100, age : 5, iconIndex : 1 });
    Goals.insert({ name : "Restaurant",   score : 750, age : 4, iconIndex : 2, isGroup : true });
    Goals.insert({ name : "Film",         score : 250, age : 4, iconIndex : 3, isGroup : true });
    
    Goals.insert({ name : "Hjerte",       score : 50,    age : 4, iconIndex : 4 });
    Goals.insert({ name : "Reise",        score : 4000,  age : 4, iconIndex : 5, isGroup : true });
    Goals.insert({ name : "Gave",         score : 500,   age : 4, iconIndex : 6 });

    Goals.insert({ name : "Cash", value : { displayText : "20" },  score : 40, age : 4, iconIndex : 7 });
    Goals.insert({ name : "Cash", value : { displayText : "50" },  score : 100, age : 4, iconIndex : 8 });
    Goals.insert({ name : "Cash", value : { displayText : "100" }, score : 200, age : 4, iconIndex : 9 });
    Goals.insert({ name : "Cash", value : { displayText : "200" }, score : 400, age : 4, iconIndex : 10 });
    Goals.insert({ name : "Cash", value : { displayText : "500" }, score : 1000, age : 4, iconIndex : 11 });

    Goals.insert({ name : "Spilletid", value : { displayText : "60" }, score : 160, age : 4, iconIndex : 12 });
    Goals.insert({ name : "Spilletid", value : { displayText : "30" }, score : 80,  age : 4, iconIndex : 13 });

    Goals.insert({ name : "Leke",         score : 120,  age : 4, iconIndex : 14 });
    Goals.insert({ name : "Godteri",      score : 80,   age : 4, iconIndex : 15 });
  }
});
