import { Meteor } from 'meteor/meteor';
import { Mongo } from 'meteor/mongo';
import { check } from 'meteor/check';

import { Groups } from '../models/group.js';
import { Profiles } from '../models/profile.js';
import { Tasks } from '../models/task.js';
import { Goals } from '../models/goal.js';

Meteor.methods({
  'groups.insert'(groupName){
      Groups.insert({
        name : groupName
      });
  },
  'profile.update'(profileId, fields) {
      Profiles.update({_id: profileId}, 
      { $set: fields}
        
      );
  },
  'profile.insert'(groupId, profile) {
    check(profile.name, String);
    //check(profile.birthDate, String);
    //check(profile.avatarIndex, Number);
    //check(profile.colorIndex, Number);

    Profiles.insert({
      groupId : groupId,
      name : profile.name,
      birthDate : profile.birthDate,
      avatar : {
        colorIndex : profile.colorIndex,
        iconIndex : profile.iconIndex
      }
    });

  },
  'profile.remove'(profileId) {
    
    Profiles.remove({
      _id:profileId
    });

  },
  'app.reset'(groupName) {
    Groups.remove({});
    Profiles.remove({});
    Tasks.remove({});
    Goals.remove({});

    var groupId = Groups.insert({ name : groupName });
    Profiles.insert({ name : "Mamma", birthDate : new Date(1980, 7, 10), groupId : groupId,
  avatar : {
        colorIndex : 1,
        iconIndex : 2
      } });
    Profiles.insert({ name : "Pappa", birthDate : new Date(1981, 7, 10), groupId : groupId,
    avatar : {
        colorIndex : 0,
        iconIndex : 1
      } 
     });


    Tasks.insert({ name : "Lage middag", score : 5, age : 5, taskIconIndex : 0 });
    Tasks.insert({ name : "Gå ut med søppel", score : 5, age : 5, taskIconIndex : 1 });
    Tasks.insert({ name : "Henge klær til tørk", score : 5, age : 4, taskIconIndex : 2 });
    Tasks.insert({ name : "Støvsuge", score : 5, age : 0, taskIconIndex : 3 });
    Tasks.insert({ name : "Brette sammen klær", score : 5, age : 1, taskIconIndex : 4 });
    Tasks.insert({ name : "Mate husdyr", score : 5, age : 2, taskIconIndex : 5 });
    Tasks.insert({ name : "Ta oppvasken", score : 5, age : 3, taskIconIndex : 6 });
    Tasks.insert({ name : "Vaske gulv", score : 5, age : 5, taskIconIndex : 7 });
    Tasks.insert({ name : "Vanne planter", score : 2, age : 2, taskIconIndex : 8 });
    Tasks.insert({ name : "Rydde", score : 5, age : 3, taskIconIndex : 9 });
    Tasks.insert({ name : "Vaske klær", score : 5, age : 4, taskIconIndex : 10 });
    Tasks.insert({ name : "Dekke bordet", score : 5, age : 3, taskIconIndex : 11 });
    Tasks.insert({ name : "Gå tur med hunden", score : 5, age : 4, taskIconIndex : 12 });

    Goals.insert({ name : "Stjerne",      score : 50,  age : 5, iconIndex : 0 });
    Goals.insert({ name : "MultiStjerne", score : 100, age : 5, iconIndex : 1 });
    Goals.insert({ name : "Restaurant",   score : 750, age : 4, iconIndex : 2, isGroup : true });
    Goals.insert({ name : "Film",         score : 250, age : 4, iconIndex : 3, isGroup : true });
    
    Goals.insert({ name : "Hjerte",       score : 50,    age : 4, iconIndex : 4 });
    Goals.insert({ name : "Reise",        score : 4000,  age : 4, iconIndex : 5, isGroup : true });
    Goals.insert({ name : "Gave",         score : 500,   age : 4, iconIndex : 6 });

    Goals.insert({ name : "Cash", value : { displayText : "20" },  score : 40, age : 4, iconIndex : 7 });
    Goals.insert({ name : "Cash", value : { displayText : "50" },  score : 100, age : 4, iconIndex : 8 });
    Goals.insert({ name : "Cash", value : { displayText : "100" }, score : 200, age : 4, iconIndex : 9 });
    Goals.insert({ name : "Cash", value : { displayText : "200" }, score : 400, age : 4, iconIndex : 10 });
    Goals.insert({ name : "Cash", value : { displayText : "500" }, score : 1000, age : 4, iconIndex : 11 });

    Goals.insert({ name : "Spilletid", value : { displayText : "60" }, score : 160, age : 4, iconIndex : 12 });
    Goals.insert({ name : "Spilletid", value : { displayText : "30" }, score : 80,  age : 4, iconIndex : 13 });

    Goals.insert({ name : "Leke",         score : 120,  age : 4, iconIndex : 14 });
    Goals.insert({ name : "Godteri",      score : 80,   age : 4, iconIndex : 15 });

  }
});

if (Meteor.isServer) {
  Meteor.publish('groups', function() {
    return Groups.find({});
  });
  Meteor.publish('profiles', function() {
    return Profiles.find({});
  });
}

Meteor.startup(() => {
  if (Groups.find({}).count() === 0) {
    // Seed groups
      var groupId = Groups.insert({ name : "Vikings", });
      if (Profiles.find({}).count() === 0) {
        Profiles.insert({ name : "Mamma", birthDate : new Date(1974, 7, 10), groupId : groupId });
        Profiles.insert({ name : "Pappa",  birthDate : new Date(1971, 8, 16), groupId : groupId, avatar: {colorIndex:0, iconIndex:1} });
      }
  }
  
  
});
