import { Meteor } from 'meteor/meteor';
import { Mongo } from 'meteor/mongo';
import { check } from 'meteor/check';

export const Goals = new Mongo.Collection('goals');

Goals.attachSchema(new SimpleSchema({
  name: {
    type: String,
  },
  age: {
    type: Number,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return 0;
      }
    }
  },
  iconIndex: {
    type: Number,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return 0;
      }
    }
  },
  score: {
    type: Number,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return 0;
      }
    }
  },
  isAssigned: {
    type: Boolean,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  goalId: { 
    type: String,
    optional: true,
    label: "Copy of goal with id",
  },
  isGroup: {
    type: Boolean,
    autoValue() { 
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  personId: {
    type: String,
    optional: true,
  },
  groupId: {
    type: String,
    optional: true,
  },
  isCompleted: {
    type: Boolean,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  isRedeemed: {
    type: Boolean,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  // XXX Should probably be called `authorId`. Is it even needed since we have
  // the `members` field?
  tasks: {
    type: [String],
    optional: true,
  },
  state: {
    type: String,
    optional: true,
    allowedValues: [
      'unassigned',
      'assigned',
      'done',
    ],
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return Goals.simpleSchema()._schema.state.allowedValues[0];
      }
    },
  },
  createdAt: {
    type: Date,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert) {
        return new Date();
      } else {
        this.unset();
      }
    },
  },
  // XXX Inconsistent field naming
  modifiedAt: {
    type: Date,
    optional: true,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isUpdate) {
        return new Date();
      } else {
        this.unset();
      }
    }
  }}));


Goals.allow({
  insert(userId, doc) {
    return allowIsBoardMember(userId, Boards.findOne(doc.boardId));
  },
  update(userId, doc) {
    return allowIsBoardMember(userId, Boards.findOne(doc.boardId));
  },
  remove(userId, doc) {
    return allowIsBoardMember(userId, Boards.findOne(doc.boardId));
  },
  fetch: ['boardId'],
});


Goals.mutations({
  assignMember(memberId) {
    return { $addToSet: { members: memberId }};
  },

  unassignMember(memberId) {
    return { $pull: { members: memberId }};
  },

  toggleMember(memberId) {
    if (this.members && this.members.indexOf(memberId) > -1) {
      return this.unassignMember(memberId);
    } else {
      return this.assignMember(memberId);
    }
  }
});