import { Meteor } from 'meteor/meteor';
import { Mongo } from 'meteor/mongo';
import { check } from 'meteor/check';

export const Tasks = new Mongo.Collection('tasks');

Tasks.attachSchema(new SimpleSchema({
  name: {
    type: String,
  },
  age: {
    type: Number,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return 0;
      }
    }
  },
  isAssigned: {
    type: Boolean,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  isApproved: {
    type: Boolean,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  isGroupTask: {
    type: Boolean,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  taskIconIndex: {
    type: Number,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return 0;
      }
    }
  },
  score: {
    type: Number,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return 0;
      }
    }
  },
  personId: {
    type: String,
    optional: true,
  },
  boardId: {
    type: String,
    optional: true,
    label: "Present on board",
  },
  taskId: { 
    type: String,
    optional: true,
    label: "Copy of task with id",
  },
  clientId: {
    type: String,
    optional: true,
    label: "Identifier set at client",
  },
  boardCount: {
    type: Number,
    optional: true,
    label: "Number of copies use in different boards",
  },
  isCompleted: {
    type: Boolean,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  isGoaled: {
    type: Boolean,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return false;
      }
    }
  },
  // XXX Should probably be called `authorId`. Is it even needed since we have
  // the `members` field?
  members: {
    type: [String],
    optional: true,
  },
  state: {
    type: String,
    optional: true,
    allowedValues: [
      'unassigned',
      'assigned',
      'done',
    ],
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return Tasks.simpleSchema()._schema.state.allowedValues[0];
      }
    },
  },
  createdAt: {
    type: Date,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert) {
        return new Date();
      } else {
        this.unset();
      }
    },
  },
  // XXX Inconsistent field naming
  modifiedAt: {
    type: Date,
    optional: true,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isUpdate) {
        return new Date();
      } else {
        this.unset();
      }
    }
  }}));


Tasks.allow({
  insert(userId, doc) {
    return allowIsBoardMember(userId, Boards.findOne(doc.boardId));
  },
  update(userId, doc) {
    return allowIsBoardMember(userId, Boards.findOne(doc.boardId));
  },
  remove(userId, doc) {
    return allowIsBoardMember(userId, Boards.findOne(doc.boardId));
  },
  fetch: ['boardId'],
});


Tasks.mutations({
  assignMember(memberId) {
    return { $addToSet: { members: memberId }};
  },

  unassignMember(memberId) {
    return { $pull: { members: memberId }};
  },

  toggleMember(memberId) {
    if (this.members && this.members.indexOf(memberId) > -1) {
      return this.unassignMember(memberId);
    } else {
      return this.assignMember(memberId);
    }
  }
});