import React, { Component, PropTypes } from 'react';
import ReactDOM from 'react-dom';
import moment from 'moment';
 
// Profile component - represents a single profile item
export default class Profile extends Component {
  constructor(props) {
    super(props);
 
    this.state = {
      editing: false,
      selectedIndex: this.props.profile.avatar.iconIndex,
      selectedColorIndex: this.props.profile.avatar.colorIndex,
      colors: ["#01AB9E","#E04D83","#F8D52C", "#F7A511", "#94C123", "#D73F24 "],
      months: ["januar","februar","mars","april","mai","juni","juli","august","september","oktober","november","desember"],
      years: this.getYears(),
      age:moment().diff(this.props.profile.birthDate, "years"),
      selectedYear:this.props.profile.birthDate.getFullYear(),
      selectedMonth: this.props.profile.birthDate.getMonth(),
      selectedDay: this.props.profile.birthDate.getDate()

    };
    console.log(this.state);
    console.log(this.props.profile);
  }
  render() {
    return (
      <li className={"task-item"+(this.state.editing ? " editing-profile" : '')}>
        <div className="item-flex">
          <div className="img-flex">
                <div  className="img-container" style={this.props.profile.avatar.colorIndex >0 ? {backgroundColor: this.state.colors[this.props.profile.avatar.colorIndex]} : null} >
                <img src={"icons/Avatar_"+this.props.profile.avatar.iconIndex+".png"} className="img-icon" />           
                </div>
          </div>
          <div className="name-flex">
                {this.props.profile.name} 
          </div>
          <div className="points-flex eq">
                {this.calculateAge(this.props.profile.birthDate)} år
                <i className="material-icons edit" onClick={this.taskEdit.bind(this)} ref="editTask">mode_edit</i>
                <i className="material-icons delete" onClick={this.taskDelete.bind(this)} ref="deleteTask" >delete</i>
          </div>

          
        </div>
      {this.state.editing &&
      
      <div ref="taskEdit" >
         <form id="add-task-form" onSubmit={this.handleSubmit.bind(this)}>
              <label htmlFor="addtaskTitle">Navn</label>
        <input type="text" 
               name="addtaskTitle"               
               maxLength="25"
               ref="addtaskTitle"
               className="add-task-text"
               defaultValue= {this.props.profile.name}
               required />
               
         <label htmlFor="yearSelect">Fødelsdag</label>
         <div className="center">
             <select name="yearSelect" ref="yearSelect" id="add-profile-year" onChange={this.yearSelected.bind(this)} required>
               
              {this.populateYears()}
  
            </select>
            
             <select name="monthSelect" ref="monthSelect" id="add-profile-month" onChange={this.monthSelected.bind(this)} required>
               
              {this.populateMonths()}
  
            </select>
            <select name="daySelect" ref="daySelect" id="add-profile-day" onChange={this.daySelected.bind(this)}required>
               
              {this.populateDays()}
  
            </select>
            <div ref="ageLine"></div>
              { "Det betyr at du er "+this.state.age+" år"}
            </div>
        <label htmlFor="addtaskIcon">Bilde</label>
        <div className="icon-container" onChange={this.setIcon.bind(this)}>
            {this.renderIcons()}
        </div>
        <label htmlFor="addColor">Farge</label>
        <div className="color-container" onChange={this.setColorIndex.bind(this)}>
            {this.renderColors()}
        </div>
        
       
      <div className="center" >
         
              <input type="submit" value="Submit" className="submit-btn"/>
      </div>
       </form>
      </div>
      }
      </li>
    );
  }

  taskEdit(event){
    this.state.editing ? this.setState({editing:false, selectedIndex: this.props.profile.avatar.iconIndex, selectedColorIndex: this.props.profile.avatar.colorIndex,selectedMonth:null,selectedDay:null,
       age: moment().diff(this.props.profile.birthDate, "years"),
       selectedYear:this.props.profile.birthDate.getFullYear(),
      selectedMonth: this.props.profile.birthDate.getMonth(),
      selectedDay: this.props.profile.birthDate.getDate()
      
    }) : this.setState({editing:true});
    
  }

  taskDelete(event){    
    Meteor.call('profile.remove', this.props.profile._id);
  }
  handleSubmit(event){
   event.preventDefault();
   const target = event.target;
   const profileId = this.props.profile._id;
   const fields = {
     name: target.addtaskTitle.value,
     birthDate: new Date(this.state.selectedYear, this.state.selectedMonth,this.state.selectedDay),
     avatar: {
       iconIndex:this.state.selectedIndex,
       colorIndex:this.state.selectedColorIndex
     }
   }
  

   
   Meteor.call('profile.update', profileId, fields);
   this.setState({editing:false});

  }

 getYears(){
   
   let currentYear = new Date(Date.now()).getFullYear();
   let years = [];
  for(let i = currentYear; i >= currentYear-110;i-- ){
    years.push(i);
  }
  return years;
 }

 yearSelected(event){
this.setState({selectedYear: event.target.value, age: this.calculateEditAge() }); 


}
monthSelected(event){
this.setState({selectedMonth: parseInt(event.target.value), age: this.calculateEditAge() }); 


}

daySelected(event){
this.setState({selectedDay: event.target.value, age: this.calculateEditAge()}); 

}
 
populateYears(){
  if(this.state.years.length > 0){
    return this.state.years.map((year, index) =>(
      
      <option key={index} value={year} selected={this.props.profile.birthDate.getFullYear() == year ? "selected" : null} >{year}</option>
    ));

    
  }
}


populateMonths(){
  if(this.state.months.length > 0){
    return this.state.months.map((month, index)=>(
        <option key={month} value={index} selected={this.props.profile.birthDate.getMonth() == index ? "selected" : null}>{month}</option>
    ));
  }
}

populateDays(){

if( (this.state.selectedYear != null || this.state.selectedYear != undefined) 
    && (this.state.selectedMonth != null || this.state.selectedMonth != undefined) ){
      
      let dayIterator = moment( this.state.selectedYear+"-"+(this.state.selectedMonth+1).toString(),"YYYY-M").daysInMonth();
      
      let days = [];
     for(let i =1; i <= dayIterator;i++){
       days.push(
         <option key={i} value={i} selected={i === this.props.profile.birthDate.getDate() ? "selected": null}>{i}</option>
       );
     }
     return days;
    }
}
 calculateEditAge(){
   const yearbox = ReactDOM.findDOMNode(this.refs.yearSelect);
   const monthbox = ReactDOM.findDOMNode(this.refs.monthSelect);
   const daybox = ReactDOM.findDOMNode(this.refs.daySelect);
   let year = yearbox.options[yearbox.selectedIndex].value;
   let month = monthbox.options[monthbox.selectedIndex].value;
   let day = daybox.options[daybox.selectedIndex].value;

  
    console.log("Day value "+day);

 if( (year != null || year != '' ) &&
      (month != null || month != '' ) &&
      (day != null || day != '' )  ){
        const birthdate = new Date(year, month, day)
        return moment().diff(birthdate, 'years');
          
 }
  else return 0;
  }
  sliderReader(event){
 ReactDOM.findDOMNode(this.refs.scoreOutput).value = ReactDOM.findDOMNode(this.refs.edittaskScore).value;
  }
  calculateAge(birthdate){

      return moment().diff(birthdate, 'years');
  }
renderIcons(){
   return this.props.icons.map((icon) => (
     <label key={"icon-"+icon.iconIndex} >
    <input type="radio" name="task-icon" ref={"icon-"+icon.iconIndex} value={icon.iconIndex} defaultChecked={this.checkSelectedIcon(icon.iconIndex)} />
    <div  className="img-container" >
      <img src={icon.path} className="img-icon" />
    </div>
    </label>
     
     ));
}
renderColors(){
  let colors = this.state.colors;
  let result =[];
  for(let i=0;i<colors.length;i++){
      result.push(
        <label key={"color-"+i }>
    <input type="radio" name="profile-color" ref={"color-"+i} value={i} defaultChecked={i== this.props.profile.avatar.colorIndex ? "checked" : null} />
    <div  className="img-container" style={{backgroundColor: colors[i] }} >
      
    </div>
    </label>
      );
  }
  
   return result;
}
checkSelectedIcon(iconIndex){

  return iconIndex == this.state.selectedIndex ? "checked" :  null;
}

checkSelectedColor(colorIndex){
  return colorIndex === this.state.selectedColorIndex ? "checked" :  null;
}
setIcon(event){
this.setState({selectedIndex:event.target.value});
}
setColorIndex(event){
this.setState({selectedColorIndex:event.target.value});
}


}
 
