﻿using System;
using System.ComponentModel;
using Constants;
using UnityEngine;

namespace Data
{
    public class Person : INotifyPropertyChanged
    {
        private string _id;
        private string _name;
        private DateTime _dateOfBirth;
        private Color _color;
        private Sprite _avatar;
        private int _experience;
        private int _level;

        private bool _isUpdateEnabled = true;
        public bool IsUpdateEnabled
        {
            get { return _isUpdateEnabled; }
            set { _isUpdateEnabled = value; }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public string Id
        {
            get { return _id; }
            set
            {
                if (_id != value)
                {
                    _id = value;
                    OnPropertyChanged(PersonProperties.Id);
                }
            }
        }

        public string Name
        {
            get { return _name; }
            set
            {
                if (_name != value)
                {
                    _name = value;
                    OnPropertyChanged(PersonProperties.Name);
                }
            }
        }

        public DateTime DateOfBirth
        {
            get { return _dateOfBirth; }
            set
            {
                if (_dateOfBirth != value)
                {
                    _dateOfBirth = value;
                    OnPropertyChanged(PersonProperties.DateOfBirth);
                }
            }
        }

        public Color Color
        {
            get { return _color; }
            set
            {
                if (_color != value)
                {
                    _color = value;
                    OnPropertyChanged(PersonProperties.Color);
                }
            }
        }

        public Sprite Avatar
        {
            get { return _avatar; }
            set
            {
                if (_avatar != value)
                {
                    _avatar = value;
                    OnPropertyChanged(PersonProperties.Avatar);
                }
            }
        }

        public int Experience
        {
            get { return _experience; }
            set
            {
                if (_experience != value)
                {
                    _experience = value;
                    OnPropertyChanged(PersonProperties.Experience);

                    Level = _experience / 25;
                }
            }
        }

        public int Level
        {
            get { return _level; }
            private set
            {
                if (_level != value)
                {
                    _level = value;
                    OnPropertyChanged(PersonProperties.Level);
                }
            }
        }

        protected void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null && _isUpdateEnabled)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
