﻿using System.ComponentModel;
using Constants;
using Data.Enums;
using UnityEngine;

namespace Data
{
    public class Task : INotifyPropertyChanged
    {
        private string _id;
        private int _score;
        private string _taskName;
        private Sprite _taskIcon;
        private Person _person;
        private AgeGroup _age;
        private bool _isCompleted;
        private bool _isGroupTask;
        private bool _hasSettled;
        private bool _isApproved;

        public bool IsUpdateEnabled { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        public WeekDays RepeatingDays { get; set; }
        public bool IsDirty { get; set; }
        public string ClientId { get; set; }
        public bool IsGoaled { get; set; }

        public string Id
        {
            get { return _id; }
            set
            {
                if (_id != value)
                {
                    _id = value;
                    OnPropertyChanged(TaskProperties.Id);
                }
            }
        }

        public bool HasSettled
        {
            get { return _hasSettled; }
            set
            {
                if (_hasSettled != value)
                {
                    _hasSettled = value;
                    OnPropertyChanged(TaskProperties.HasSettled);
                }
            }
        }

        public bool IsApproved
        {
            get { return _isApproved; }
            set
            {
                if (_isApproved != value)
                {
                    _isApproved = value;
                    OnPropertyChanged(TaskProperties.IsApproved);
                }
            }
        }

        public int Score
        {
            get { return _score; }
            set
            {
                if (_score != value)
                {
                    _score = value;
                    OnPropertyChanged(TaskProperties.Score);
                }
            }
        }

        public string TaskName
        {
            get { return _taskName; }
            set
            {
                if (_taskName != value)
                {
                    _taskName = value;
                    OnPropertyChanged(TaskProperties.TaskName);
                }
            }
        }

        public Sprite TaskIcon
        {
            get { return _taskIcon; }
            set
            {
                if (_taskIcon != value)
                {
                    _taskIcon = value;
                    OnPropertyChanged(TaskProperties.TaskIcon);
                }
            }
        }

        public Person Person
        {
            get { return _person; }
            set
            {
                if (_person != value)
                {
                    _person = value;
                    OnPropertyChanged(TaskProperties.Person);
                }
            }
        }

        public bool IsCompleted
        {
            get { return _isCompleted; }
            set
            {
                if (_isCompleted != value)
                {
                    _isCompleted = value;
                    OnPropertyChanged(TaskProperties.IsCompleted);
                }
            }
        }

        public string TaskIconUrl
        {
            get { return _taskName; }
            set
            {
                if (_taskName != value)
                {
                    _taskName = value;
                    OnPropertyChanged(TaskProperties.TaskName);
                }
            }
        }

        public AgeGroup Age
        {
            get { return _age; }
            set
            {
                if (_age != value)
                {
                    _age = value;
                    OnPropertyChanged(TaskProperties.Age);
                }
            }
        }

        public bool IsGroupTask
        {
            get { return _isGroupTask; }
            set
            {
                if (_isGroupTask != value)
                {
                    _isGroupTask = value;
                    OnPropertyChanged(TaskProperties.IsGroupTask);
                }
            }
        }


        public Task() { }

        public Task(Task task)
        {
            _id = task._id;
            _score = task._score;
            _taskName = task._taskName;
            _taskIcon = task._taskIcon;
            _person = task._person;
            _age = task._age;
            _isCompleted = task._isCompleted;
            _isGroupTask = task._isGroupTask;
            _hasSettled = task._hasSettled;
            _isApproved = task._isApproved;
        }

        protected void OnPropertyChanged(string propertyName)
        {
            IsDirty = true;
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null && IsUpdateEnabled)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
