﻿using UnityEngine;
using UnityEngine.EventSystems;

public class TouchInput : MonoBehaviour
{
    private Vector3 startPoint;
    private Vector3 endPoint;

    public enum SwipeDirection { Up, Down, Left, Right };

    public void Update()
    {
        // Block UI calls
        if (EventSystem.current.IsPointerOverGameObject()) return;

        // Detect touch begin
        if (Input.GetMouseButtonDown(0) || (Input.touchCount > 0 &&
            Input.GetTouch(0).phase == TouchPhase.Began))
        {
            // Check if touching a task ball -> cancel touch action
            RaycastHit2D hit = Physics2D.Raycast(
                Camera.main.ScreenToWorldPoint(Input.mousePosition), Vector2.zero);
            if (hit.transform == null || hit.transform.GetComponent<TaskBallController>() == null)
            {
                startPoint = Input.mousePosition;
                endPoint = Input.mousePosition;
            }
        }

        // Detect thouch moved
        if ((Input.GetMouseButton(0) || (Input.touchCount > 0
            && Input.GetTouch(0).phase == TouchPhase.Moved)) &&
            !startPoint.Equals(Vector3.zero))
        {
            endPoint = Input.mousePosition;
            float dx = endPoint.x - startPoint.x;
            float dy = endPoint.y - startPoint.y;

            if (Mathf.Abs(dx) > Constants.Touch.swipeTolerance ||
                Mathf.Abs(dy) > Constants.Touch.swipeTolerance)
            {
                SwipeDirection direction;
                if (Mathf.Abs(dx) > Mathf.Abs(dy))
                    direction = (dx > 0) ? SwipeDirection.Right : SwipeDirection.Left;
                else
                    direction = (dy > 0) ? SwipeDirection.Up : SwipeDirection.Down;
                Swipe(direction);
            }
        }

        // Detect touch ended
        if (Input.GetMouseButtonUp(0) || (Input.touchCount > 0 &&
            Input.GetTouch(0).phase == TouchPhase.Ended))
        {
            startPoint = Vector3.zero;
            endPoint = Vector3.zero;
        }
    }

    private void Swipe(SwipeDirection direction)
    {
        CameraController cameraController = Camera.main.GetComponent<CameraController>();
        if (direction == SwipeDirection.Up)
            cameraController.ScrollDown();
        else if (direction == SwipeDirection.Down)
            cameraController.ScrollUp();
    }
}
