﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

/*
 * Controls the functionality of the homework GUI
 */
public class HomeworkGUIController : MonoBehaviour
{
    public GameObject tabPrefab;
    public GameObject tabFinishPrefab;
    public GameObject linkPrefab;
    public GameObject particlePrefab;
    public Sprite checkIcon;
    public GameObject backdrop;

    public Toggle checkbox;
    public Text taskText;
    public GameObject finishButton;
    public Image homeworkIcon;
    public GameObject avatar;

    public GameObject backgroundBodyFinished;
    public GameObject backgroundBody;
    public GameObject backgroundBorder;

    // Feedback buttons
    public GameObject easyButton;
    public GameObject adequateButton;
    public GameObject hardButton;

    private int activeTabIndex = 0;
    private HomeworkTask[] homework;
    private bool hasClickedFeedback = false;
    private bool hasClickedTimeSpent = false;
    private bool hasClickedATab = false;
    private bool setup = true;

    private GameObject OriginTaskIcon;
    private List<GameObject> GUITabs = new List<GameObject>();
    private TabProperties[] tabProperties = new TabProperties[] { new TabProperties(-195, 205, new Color32(0xab, 0x45, 0x67, 0xff)), new TabProperties(-275, 95, new Color32(0x48, 0x68, 0x82, 0xff)), new TabProperties(-293, -35, new Color32(0xf8, 0xb1, 0x33, 0xff)), new TabProperties(-240, -155, new Color32(0x4a, 0x87, 0x65, 0xff)) };
    private TabProperties[] urlTabProperties = new TabProperties[] { new TabProperties(275, 0, new Color32()), new TabProperties(293, -35, new Color32()), new TabProperties(240, -155, new Color32()) };

    public void SetHomework(HomeworkTask[] homework)
    {
        this.homework = homework;
        PopulateGUI();
    }

    public void SetOriginTaskIcon(GameObject origin)
    {
        this.OriginTaskIcon = origin;
    }

    private void PopulateGUI()
    {
        backgroundBody.GetComponent<Image>().alphaHitTestMinimumThreshold = 1;
        backgroundBorder.GetComponent<Image>().alphaHitTestMinimumThreshold = 1;
        backgroundBodyFinished.GetComponent<Image>().alphaHitTestMinimumThreshold = 1;
        tabPrefab.GetComponent<Image>().alphaHitTestMinimumThreshold = 1;
        tabFinishPrefab.GetComponent<Image>().alphaHitTestMinimumThreshold = 1;
        linkPrefab.GetComponent<Image>().alphaHitTestMinimumThreshold = 1;

        // Create GUI tab for each homework task
        foreach (HomeworkTask homeworkTask in homework)
        {
            var taskTabToAdd = Instantiate(tabPrefab);
            GUITabs.Add(taskTabToAdd);
            taskTabToAdd.GetComponentInChildren<Text>().text = "" + GUITabs.Count;
        }

        // Set the finished tab
        var finishTab = Instantiate(tabFinishPrefab);
        GUITabs.Add(finishTab);
        finishTab.GetComponentInChildren<Text>().text = "";
        finishTab.SetActive(false);


        // Position and give index to tabs
        for (int i = 0; i < GUITabs.Count; i++)
        {
            GUITabs[i].transform.SetParent(transform, false);
            GUITabs[i].transform.localPosition = new Vector3(tabProperties[i].GetX(), tabProperties[i].GetY(), 0);
            GUITabs[i].GetComponent<HomeworkGUITabController>().SetIndex(i);
            GUITabs[i].GetComponent<HomeworkGUITabController>().SetColor(tabProperties[i].GetColor());
        }
        SetUpTestUser();

        SetActiveTab(activeTabIndex);

        setup = false;
    }

    private void SetUpTestUser()
    {
        // Locate the gameobject that keeps the pool names.
        GameObject poolNameContainer = GameObject.Find("TopCanvas/PoolNames");
        // Extract the name of the right-most user and set it in the GUI.
        Transform rightMostUsername = poolNameContainer.transform.GetChild(poolNameContainer.transform.childCount - 1);
        avatar.GetComponentInChildren<Text>().text = rightMostUsername.GetComponent<Text>().text;

        // Locate the gameobject that keeps the pools
        GameObject poolContainer = GameObject.Find("PoolRoot");
        // Extract the right-most user's avatar and set it to the GUI.
        Transform rightMostUserAvatar = poolContainer.transform.GetChild(poolContainer.transform.childCount - 4);
        Sprite avatarSprite = rightMostUserAvatar.FindChild("Avatar/AvatarIcon").GetComponent<SpriteRenderer>().sprite;
        avatar.transform.FindChild("Image").GetComponentInChildren<Image>().sprite = avatarSprite;
    }

    private void SetActiveTabIllusion()
    {
        backgroundBody.transform.SetSiblingIndex(0);
        GUITabs[activeTabIndex].transform.SetSiblingIndex(0);
        backgroundBorder.transform.SetSiblingIndex(0);
        
        for (int i = 0; i < GUITabs.Count; i++)
        {
            if (i != activeTabIndex)
            {
                GUITabs[i].transform.SetSiblingIndex(0);
            }
        }
        backdrop.transform.SetSiblingIndex(0);

        backgroundBorder.GetComponent<Image>().color = GUITabs[activeTabIndex].GetComponent<Image>().color;
        backgroundBody.GetComponentInChildren<Text>().color = GUITabs[activeTabIndex].GetComponent<Image>().color;
    }

    public void SetHomeworkIcon(Sprite homeworkIcon)
    {
        this.homeworkIcon.sprite = homeworkIcon;
    }

    public void SetActiveTab(int tabIndex)
    {
        activeTabIndex = tabIndex;

        if (activeTabIndex <= (homework.Length - 1)) //GUITabs is always one bigger than homework because of the check button.
        {
            taskText.text = homework[activeTabIndex].GetTaskText();
            checkbox.isOn = homework[activeTabIndex].GetIsFinished();

            if(activeTabIndex != GUITabs.Count - 1)
            {
                if (homework[activeTabIndex].GetHomeworkLinks().Length > 0)
                {
                    //TODO LINKS
                    for (int i = 0; i < homework[activeTabIndex].GetHomeworkLinks().Length; i++)
                    {
                        var linkToSpawn = Instantiate(linkPrefab);

                        linkToSpawn.transform.SetParent(transform, false);
                        linkToSpawn.transform.localPosition = new Vector3(urlTabProperties[i].GetX(), urlTabProperties[i].GetY(), 0);
                        linkToSpawn.GetComponent<LinkTabController>().SetText(homework[activeTabIndex].GetHomeworkLinks()[i].GetTitle());
                        linkToSpawn.GetComponent<LinkTabController>().SetLink(homework[activeTabIndex].GetHomeworkLinks()[i].GetUrl());
                        linkToSpawn.GetComponent<LinkTabController>().SetColor(GUITabs[activeTabIndex].GetComponent<Image>().color);
                    }

                }
                else
                {
                    RemoveLinks();
                }
            }

            backgroundBody.SetActive(true);
            backgroundBodyFinished.SetActive(false);
        }
        else
        {
            backgroundBody.SetActive(false);
            backgroundBodyFinished.SetActive(true);
            RemoveLinks();
        }

        SetActiveTabIllusion();
    }

    public void ToggleCheckbox(bool toggleValue)
    {
        homework[activeTabIndex].SetIsFinished(toggleValue);

        if (AllTasksCompleted() && !setup)
        {
            GUITabs[GUITabs.Count - 1].SetActive(true);
            SetActiveTab(GUITabs.Count - 1);
        }
        else if (AllTasksCompleted() )
        {
            GUITabs[GUITabs.Count - 1].SetActive(true);
        }
        else
        {
            GUITabs[GUITabs.Count - 1].SetActive(false);
            finishButton.SetActive(false);
            hasClickedFeedback = false;
            hasClickedTimeSpent = false;
        }
    }

    private void RemoveLinks()
    {
        foreach (Transform child in transform)
        {
            if (child.tag == "Link")
            {
                Destroy(child.gameObject);
            }
        }
    }

    public void ClickFeedbackButton(FeedbackButton feedbackButtonType)
    {
        Color inActiveColor = new Color32(0xFF, 0xFF, 0xFF, 0xFF);
        Color activeColor = new Color32(0x4a, 0x87, 0x65, 0xFF);

        if (feedbackButtonType == FeedbackButton.EASY)
        {
            easyButton.GetComponent<Image>().color = activeColor;
            adequateButton.GetComponent<Image>().color = inActiveColor;
            hardButton.GetComponent<Image>().color = inActiveColor;
        }
        else if (feedbackButtonType == FeedbackButton.ADEQUATE)
        {
            easyButton.GetComponent<Image>().color = inActiveColor;
            adequateButton.GetComponent<Image>().color = activeColor;
            hardButton.GetComponent<Image>().color = inActiveColor;
        }
        else if (feedbackButtonType == FeedbackButton.HARD)
        {
            easyButton.GetComponent<Image>().color = inActiveColor;
            adequateButton.GetComponent<Image>().color = inActiveColor;
            hardButton.GetComponent<Image>().color = activeColor;
        }
        hasClickedFeedback = true;
        CheckIfFinishButtonShouldBeShown();
    }

    public void ClickTimeSpentRadioButton()
    {
        hasClickedTimeSpent = true;

        CheckIfFinishButtonShouldBeShown();
    }

    private void CheckIfFinishButtonShouldBeShown()
    {
        if(hasClickedFeedback && hasClickedTimeSpent)
        {
            finishButton.SetActive(true);
        }
    }

    public void Close()
    {
        activeTabIndex = 0;
        SetActiveTab(activeTabIndex);
        Destroy(gameObject);
    }

    public void FinishHomework()
    {
        Color avatarColor = FindRightMostUser().GetComponentInChildren<SpriteRenderer>().color;
        for (int i = 0; i < 50; i++)
        {
            var particleToSpawn = Instantiate(particlePrefab, OriginTaskIcon.transform.position, particlePrefab.transform.rotation);
            particleToSpawn.GetComponentInChildren<SpriteRenderer>().color = avatarColor;
            particleToSpawn.GetComponent<ParticleController>().SetEndTarget(OriginTaskIcon.GetComponent<HomeworkTaskController>().GetTarget());
            particleToSpawn.GetComponent<ParticleController>().SetRandomOffset(new Vector3(Random.Range(-4f, 4f), Random.Range(-4f, 4f), 0f));

        }
        Destroy(OriginTaskIcon.gameObject);
        this.Close();

    }

    private GameObject FindRightMostUser()
    {
        int todoPoolIndex = ((GameObject.Find("PoolRoot").transform.childCount - 2) / 2) - 1;

        return GameObject.Find("PoolRoot/TodoPool:" + todoPoolIndex + "/Avatar");
    }

    private void ShowFinishTab()
    {

        this.SetActiveTab(GUITabs.Count - 1);
    }

    private bool AllTasksCompleted()
    {
        foreach (HomeworkTask task in homework)
        {
            if (task.GetIsFinished() == false)
            {
                return false;
            }
        }
        return true;
    }

}
