﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

/*
 * Controls the functionality in the school class GUI
 */
public class SchoolClassController : MonoBehaviour {

    public Image backdrop;
    public GameObject messagePrefab;

    public Text schoolClassHeading;
    public GameObject messageList;
    public GameObject highScorers;
    public Image progressbar;

    public Sprite male;
    public Sprite female;

    public void SetUpWindowInformation(SchoolClass schoolClass)
    {
        this.schoolClassHeading.text = "Klasse " + schoolClass.GetName();

        foreach (string message in schoolClass.GetMessages())
        {
            var messageToSpawn = Instantiate(this.messagePrefab);
            messageToSpawn.GetComponentInChildren<Text>().text = message;
            messageToSpawn.transform.SetParent(this.messageList.transform, false);
        }
        this.progressbar.fillAmount = this.SetProgressbar(schoolClass.GetCurrentPoints(), schoolClass.GetMaxPoints() );
        this.progressbar.GetComponentInChildren<Text>().text = schoolClass.GetRewardName();

        //UserImage1
        SetUpTestUserInThePodium();

        for (int i = 1; i < 3; i++)
        {
            GameObject.Find("Highscorers/Username" + (i+1) ).GetComponent<Text>().text = schoolClass.GetTopCollectors()[i].GetName();
            Image userImage = GameObject.Find("Highscorers/UserImage" + (i + 1)).GetComponent<Image>();
            if(schoolClass.GetTopCollectors()[i].GetGender() == Gender.FEMALE)
            {
                userImage.sprite = female;
            }
            else
            {
                userImage.sprite = male;
            }
        }

    }

    private void SetUpTestUserInThePodium()
    {
        // Locate the gameobject that keeps the pool names.
        GameObject poolNameContainer = GameObject.Find("TopCanvas/PoolNames");
        // Extract the name of the right-most user and set it to the top of the leaderboard.
        Transform rightMostUsername = poolNameContainer.transform.GetChild(poolNameContainer.transform.childCount - 1);
        GameObject.Find("Highscorers/Username" + (1)).GetComponent<Text>().text = rightMostUsername.GetComponent<Text>().text;

        // Locate the gameobject that keeps the pools
        GameObject poolContainer =  GameObject.Find("PoolRoot");
        // Extract the right-most user's avatar and set it to the top of the leaderboard.
        Transform rightMostUserAvatar = poolContainer.transform.GetChild(poolContainer.transform.childCount - 4);
        Sprite avatarSprite = rightMostUserAvatar.FindChild("Avatar/AvatarIcon").GetComponent<SpriteRenderer>().sprite;
        GameObject.Find("Highscorers/UserImage" + (1)).GetComponent<Image>().sprite = avatarSprite;
    }

    private float SetProgressbar(float currentPoints, float maxPoints)
    {
        // Max fill = 0.4
        float fillAmount = (currentPoints / maxPoints) * 0.4f;
        if (fillAmount > 0.4)
        {
            fillAmount = 0.4f;
        }

        return fillAmount;
    }

    public void ExtraTaskButton()
    {
        GameObject.Find("Startup/HomeworkTaskManager").GetComponent<HomeworkTaskManager>().SpawnExtraTask();
        Close();
    }

    public void Close()
    {
        Destroy(gameObject);
    }
}
