﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Orchestration
{
    /// <summary>
    /// Ensures that components are started in the correct order
    /// </summary>
    public class StartUp : MonoBehaviour
    {
        public bool Verbose;

        private List<IStartupComponent> _orderedComponents;

        public IEnumerator Start()
        {
            FetchChildComponents();
            Log("Components: " + string.Join(", ", _orderedComponents.Select(c => c.GetType().Name + ":" + c.InitOrder).ToArray()));

            Log("Initializing components");
            foreach (var component in _orderedComponents)
            {
                Log(" -- Starting " + component.GetType().Name);
                component.Initialize();
                while (!component.IsInitComplete)
                {
                    yield return null;
                }
            }

            Debug.Log("Startup complete");
        }

        // TODO kjellre 2017-03-11 Consider doing an ordered unload as well

        private void FetchChildComponents()
        {
            var children = new List<GameObject>();
            for (var i = 0; i < transform.childCount; i++)
            {
                children.Add(transform.GetChild(i).gameObject);
            }

            _orderedComponents = children
                .SelectMany(go => go.GetComponents<IStartupComponent>())
                .OrderBy(c => c.InitOrder)
                .ToList();
        }

        private void Log(string message)
        {
            if (Verbose)
            {
                Debug.Log(message);
            }
        }
    }
}
