﻿using System;
using System.Collections.Generic;
using Data;
using Manage.Services.Synchronizers;
using Orchestration;
using UnityEngine;

public class PoolController : MonoBehaviour, IStartupComponent
{
    public GameObject todoPoolPrefab;
    public GameObject rewardPoolPrefab;
    public GameObject FamilyPoolPrefab;
    public PersonSynchronizer personSynchronizer;

    public Dictionary<Person, GameObject> todoPools;
    public Dictionary<Person, GameObject> rewardPools;

    private static float minX;
    private static float maxX;
    private static float poolWidth;
    private static float poolHeight;
    private static float spacing;

    private bool _isIndividualMode;
    private GameObject _familyPool;
    private Transform _poolRoot;

    public RewardPool FamilyPool
    {
        get { return _familyPool.GetComponent<RewardPool>(); }
    }

    #region IStartupComponent

    public int InitOrder { get { return 6; } }
    public bool IsInitComplete { get; private set; }

    public void Initialize()
    {
        var poolRoot = GameObject.Find("PoolRoot");
        _poolRoot = poolRoot == null ? null : poolRoot.transform;

        var persons = personSynchronizer.Persons;
        spacing = 0.5f;
        todoPools = new Dictionary<Person, GameObject>();
        rewardPools = new Dictionary<Person, GameObject>();
        maxX = Camera.main.orthographicSize * Camera.main.aspect;
        minX = -maxX;
        minX -= spacing / 2; // TODO: fix hotfix
        poolWidth = (maxX - minX) / persons.Count - spacing;
        poolHeight = Camera.main.orthographicSize * Constants.Pools.poolHeight;

        for (var i = 0; i < persons.Count; i++)
        {
            CreatePersonPools(persons[i], i);
        }
        CreateFamilyPool();

        IsInitComplete = true;
    }

    #endregion

    public GameObject GetTodoPool(Person person)
    {
        return todoPools[person];
    }

    public GameObject GetRewardPool(Person person)
    {
        return rewardPools == null ? null : rewardPools[person];
    }

    private void CreatePersonPools(Person person, int index)
    {
        // Instantiate TodoPool GameObject
        var todoPool = Instantiate(todoPoolPrefab);
        todoPool.transform.parent = _poolRoot;
        todoPool.name = "TodoPool:" + index;
        var todoScript = todoPool.GetComponent<TodoPool>();
        var todoBg = todoPool.transform.FindChild("Pool");
        var todoAvatar = todoPool.transform.FindChild("Avatar");
        todoScript.Person = person;

        // Instantiate reward pool GameObject
        var rewardPool = Instantiate(rewardPoolPrefab);
        rewardPool.transform.parent = _poolRoot;
        rewardPool.name = "RewardPool:" + index;
        var rewardScript = rewardPool.GetComponent<RewardPool>();
        var rewardBg = rewardPool.transform.FindChild("Pool");
        var rewardAvatar = rewardPool.transform.FindChild("Avatar");
        rewardScript.isHidden = true;
        rewardScript.Person = person;

        // Set scale of todo pool
        float todoWidth = todoScript.poolBackground.bounds.size.x;
        float rewardWidth = rewardScript.poolBg.bounds.size.x;
        float todoHeight = todoScript.poolBackground.bounds.size.y;
        todoBg.transform.localScale = new Vector2(poolWidth / todoWidth, poolHeight / todoHeight);
        rewardBg.transform.localScale = new Vector2(poolWidth / rewardWidth, rewardBg.transform.localScale.y);

        // Set position of pool
        float poolX = minX + (poolWidth + spacing / 2f) * (index + 0.5f) + spacing;
        float poolY = poolHeight / 2f - Camera.main.orthographicSize * 0.9f;
        todoPool.transform.position = new Vector2(poolX, poolY);
        rewardPool.transform.position = new Vector2(poolX, -8.4f); // TODO fix hotfix

        rewardScript.Hide();

        todoPools[person] = todoPool;
        rewardPools[person] = rewardPool;
    }

    private void CreateFamilyPool()
    {
        _familyPool = Instantiate(FamilyPoolPrefab);
        _familyPool.transform.parent = _poolRoot;
        _familyPool.name = "FamilyPool";

        var poolScript = _familyPool.GetComponent<RewardPool>();
        poolScript.isFamilyPool = true;
        poolScript.isHidden = false;
    }

    public void ToggleIndividualMode()
    {
        _isIndividualMode = !_isIndividualMode;
        if (_isIndividualMode)
        {
            _familyPool.GetComponent<RewardPool>().Hide();
            foreach (var rewardPool in rewardPools)
            {
                rewardPool.Value.GetComponent<RewardPool>().Show();
            }
        }
        else
        {
            foreach (var rewardPool in rewardPools)
            {
                rewardPool.Value.GetComponent<RewardPool>().Hide();
            }
            _familyPool.GetComponent<RewardPool>().Show();
        }
    }
}
