﻿using System.Collections.Generic;
using Data;
using Extensions;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;
using Data.Enums;
using Manage.Services.Synchronizers;

public class TaskMenu : MonoBehaviour
{
    public Image taskBackground;
    public Image taskIcon;
    public Image avatarBackground;
    public Image avatarIcon;
    public Image approveTask;
    public Image repeatingTask;
    public Image group;
    public Image age;
    public Text taskText;
    public Text personName;
    public Text repeatingDays;
    public Sprite approvedSprite;
    public Sprite repeatingEnabled;
    public Sprite repeatingDisabled;
    public GameObject repeatingTaskMenuPrefab;

    private Task task;
    private GameObject repeatingTaskMenu;

    public TaskSynchronizer TaskSynchronizer { get; set; }

    public void Awake()
    {
        taskBackground.color = Constants.Colors.undefinedBackgroundColor;
    }

    public void Update()
    {
        if (Input.GetMouseButtonDown(0))
        {
            var pointer = new PointerEventData(EventSystem.current);
            pointer.position = Input.mousePosition;
            var raycastResults = new List<RaycastResult>();
            EventSystem.current.RaycastAll(pointer, raycastResults);
            if (raycastResults.Count == 0)
            {
                Destroy(gameObject);
            }
        }
    }

    public void OnDestroy()
    {
        if (repeatingTaskMenu != null)
            Destroy(repeatingTaskMenu);
    }

    public void CompleteTask()
    {
        task.IsCompleted = true;
        Destroy(gameObject);
    }

    public void Cancel()
    {
        Destroy(gameObject);
    }

    public void DeleteTask()
    {
        TaskSynchronizer.DeleteTask(task);
        Destroy(gameObject);
    }

    public void ApproveTask()
    {
        if (task.IsApproved) return;
        task.IsApproved = true;
        Destroy(gameObject);
    }

    public void OpenRepeatingTaskMenu()
    {
        if (repeatingTaskMenu == null)
        {
            repeatingTaskMenu = Instantiate(repeatingTaskMenuPrefab);
            var canvas = GameObject.Find("Canvas");
            var repeatingTaskMenuScript = repeatingTaskMenu.GetComponent<RepeatingTaskMenu>();
            repeatingTaskMenuScript.Task = task;
            repeatingTaskMenuScript.TaskMenu = this;
            repeatingTaskMenu.transform.SetParent(canvas.transform, false);
        }
        else
        {
            Destroy(repeatingTaskMenu);
        }
    }

    public Task Task
    {
        get { return task; }
        set
        {
            task = value;
            InitTaskMenu();
        }
    }

    private void InitTaskMenu()
    {
        taskIcon.sprite = task.TaskIcon;
        if (task.IsApproved)
            approveTask.sprite = approvedSprite;
        if (taskText != null)
            taskText.text = task.TaskName;
        if (task.Person != null)
        {
            taskBackground.color = task.Person.Color;
            avatarIcon.sprite = task.Person.Avatar;
            if (task.IsCompleted)
                avatarBackground.color = task.Person.Color;
            if (personName != null)
                personName.text = task.Person.Name;
        }
        if (age != null)
            age.sprite = Task.Age.GetSprite();
        if (group != null)
        {
            var spriteIndex = Task.IsGroupTask ? 1 : 0;
            group.sprite = GameLogic.Instance.groupIcons[spriteIndex];
        }
        UpdateRepeatingInterface();
    }

    public void UpdateRepeatingInterface()
    {
        if (repeatingTask == null)
            return;
        if (task.RepeatingDays != WeekDays.None)
        {
            repeatingTask.sprite = repeatingEnabled;
            repeatingDays.text = "Dager:\n";
            for (int i = 0; i < Constants.UI.weekDays.Count; i++)
            {
                WeekDays weekDay = (WeekDays)(1 << i);
                var isRepeating = (task.RepeatingDays & weekDay) != 0;
                if (isRepeating)
                {
                    repeatingDays.text += Constants.UI.weekDays[i];
                    if (i != Constants.UI.weekDays.Count - 1)
                        repeatingDays.text += ", ";
                }
            }
        }
        else
        {
            repeatingTask.sprite = repeatingDisabled;
            repeatingDays.text = "";
        }
    }
}
