﻿using System;
using System.Collections.Generic;
using Data;
using Manage.Services.Synchronizers;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class PersonEditMenu : MonoBehaviour
{
    private Person person;

    public PersonView personPreview;
    public InputField nameInput;

    public Dropdown dayDropdown;
    public Dropdown monthDropdown;
    public Dropdown yearDropdown;

    public Image backgroundColor;
    public List<Image> avatarViews;
    public List<Image> colorViews;

    public PersonSynchronizer personSynchronizer;

    public static List<string> days;

    public Person Person
    {
        get { return person; }
        set
        {
            if (person != value)
            {
                person = value;
                UpdateAvatarViews();
                UpdateColorViews();

                // Check for small menu
                if (nameInput == null)
                {
                    transform.Find("Name").GetComponent<Text>().text = person.Name;
                    transform.Find("Age").GetComponent<Text>().text = 
                        GameLogic.AgeInYears(person.DateOfBirth) + " " + Constants.UI.years;
                }
                else
                {
                    personPreview.Person = person;
                    if(person.Name != null)
                        nameInput.text = person.Name;
                    InitializeBirthDateDropdowns();
                    OnDateOfBirthChange();
                }
            }
        }
    }

    // Start is called just before any of the Update methods is called the first time
    public void Start()
    {
        personSynchronizer = GameObject.Find("Server").GetComponent<PersonSynchronizer>();
        if (person == null)
            Person = new Person();
    }

    private void InitializeBirthDateDropdowns()
    {
        if (person.DateOfBirth == default(DateTime))
            person.DateOfBirth = new DateTime(DateTime.Now.Year - 20, 1, 1);
        days = new List<string>();
        for (int i = 1; i <= 31; i++)
            days.Add(i.ToString());
        var years = new List<string>();
        for (int i = DateTime.Now.Year - Constants.UI.maxAge; i < DateTime.Now.Year; i++)
            years.Add(i.ToString());

        dayDropdown.AddOptions(days);
        monthDropdown.AddOptions(Constants.UI.months);
        yearDropdown.AddOptions(years);

        dayDropdown.value = person.DateOfBirth.Day - 1;
        monthDropdown.value = person.DateOfBirth.Month - 1;
        yearDropdown.value = person.DateOfBirth.Year - DateTime.Now.Year + Constants.UI.maxAge;
    }

    private void UpdateAvatarViews()
    {
        if (person.Avatar == null)
            person.Avatar = GameLogic.Instance.avatars[0];
        int index = GameLogic.Instance.avatars.IndexOf(person.Avatar) - 1;
        foreach (var avatarView in avatarViews)
        {
            if (index < 0 || index >= GameLogic.Instance.avatars.Count)
                avatarView.gameObject.SetActive(false);
            else
            {
                avatarView.gameObject.SetActive(true);
                avatarView.sprite = GameLogic.Instance.avatars[index];
            }
            index++;
        }
    }

    private void UpdateColorViews()
    {
        if (person.Color.a == 0)
            person.Color = GameLogic.Instance.colors[0];
        int index = GameLogic.Instance.colors.IndexOf(person.Color) - 1;
        foreach (var colorView in colorViews)
        {
            if (index < 0 || index >= GameLogic.Instance.colors.Count)
                colorView.color = Color.clear;
            else
                colorView.color = GameLogic.Instance.colors[index];
            index++;
        }
        backgroundColor.color = person.Color;
    }

    public void OnNameChanged()
    {
        person.Name = nameInput.text;
        personPreview.personName.text = person.Name;
    }

    public void OnDateOfBirthChange()
    {
        int year = int.Parse(yearDropdown.options[yearDropdown.value].text);
        int month = monthDropdown.value + 1;
        var daysInMonth = DateTime.DaysInMonth(year, month);
        int day = Mathf.Min(dayDropdown.value + 1, daysInMonth);
        if (dayDropdown.options.Count != daysInMonth)
        {
            dayDropdown.ClearOptions();
            dayDropdown.AddOptions(days.GetRange(0, daysInMonth));
        }
        dayDropdown.value = day - 1;

        var date = new DateTime(year, month, day);
        person.DateOfBirth = date;
    }

    public void AvatarChange(int increment)
    {
        var avatars = GameLogic.Instance.avatars;
        int index = avatars.IndexOf(person.Avatar);
        var newIndex = index + increment;
        if (newIndex >= 0 && newIndex < avatars.Count)
        {
            person.Avatar = avatars[newIndex];
            UpdateAvatarViews();
        }
    }

    public void ColorChange(int increment)
    {
        var colors = GameLogic.Instance.colors;
        int index = colors.IndexOf(person.Color);
        var newIndex = index + increment;
        if (newIndex >= 0 && newIndex < colors.Count)
        {
            person.Color = colors[newIndex];
            UpdateColorViews();
        }
    }

    public void CompletePersonEdit()
    {
        if (person.Name != null)
        {
            personSynchronizer.InsertPerson(person);
            SceneManager.LoadScene("FamilyEditMenu");
        }
    }

    public void Destroy()
    {
        GameObject.Destroy(gameObject);
    }
}
